import mock
import pytest
from algosec.models import DeviceAllowanceState

from algosec_resilient.components.algosec_check_host_internet_connectivity import AlgoSecCheckHostInternetConnectivity
from tests.conftest import run_resilient_function


class TestAlgoSecCheckHostInternetConnectivity(object):

    @pytest.fixture
    def additional_options(self):
        return {
            'internet_connectivity_check_external_ip': 'some-ip-that-is-found-in-the-www',
            'internet_connectivity_check_service': 'some-service-like-http',
        }

    @pytest.fixture
    def tested_class(self):
        return AlgoSecCheckHostInternetConnectivity

    @pytest.mark.parametrize('device_allowance,expected_connectivity', [
        (DeviceAllowanceState.PARTIALLY_BLOCKED, 'Yes'),
        (DeviceAllowanceState.BLOCKED, 'No'),
        (DeviceAllowanceState.ALLOWED, 'Yes'),
        (DeviceAllowanceState.NOT_ROUTED, 'No'),
    ])
    @mock.patch(
        'algosec_resilient.components.algosec_client.FirewallAnalyzerAPIClient.execute_traffic_simulation_query'
    )
    def test_algosec_check_host_internet_connectivity(
            self,
            mock_execute_traffic_simulation_query,
            device_allowance,
            expected_connectivity,
            additional_options,
            tested_function,
    ):
        query_url = 'some-query-url'
        mock_execute_traffic_simulation_query.return_value = {
            'result': device_allowance,
            'query_url': query_url,
        }

        ip = '10.0.0.1'
        result = run_resilient_function(tested_function, ip)

        mock_execute_traffic_simulation_query.assert_called_once_with(
            ip,
            additional_options['internet_connectivity_check_external_ip'],
            additional_options['internet_connectivity_check_service'],
        )

        assert result == {
            'success': True,
            'artifact_ip': ip,
            'is_it_connected_to_the_internet': expected_connectivity,
            'query_url': '<a href="{}">Query Results</a>'.format(query_url),
        }



# -*- coding: utf-8 -*-

"""Generate a default configuration-file section for algosec"""

from __future__ import print_function

from textwrap import dedent


def config_section_data():
    """Produce the default configuration section for app.config,
       when called by `resilient-circuits config [-c|-u]`
    """
    config_data = dedent(
        u"""
        [algosec]
        server_ip=local.algosec.com
        username=admin
        password=algosec
        ssl_enabled=true
        isolation_request_subject=IBM Resilient Isolation Request for {}
        isolation_request_requestor=IBM Resilient
        isolation_request_requestor_email=resilient@integration.dummy
        isolation_request_description=Isolation Request initiated by the IBM Resilient Integration.
        isolation_request_template=

        internet_connectivity_check_external_ip=8.8.8.8
        internet_connectivity_check_service=*
        """
    )
    return config_data



import mock
import pytest
from resilient_circuits import FunctionResult
from resilient_circuits.action_message import FunctionException_


@pytest.fixture
def base_options():
    return {
        'algosec': {
            'server_ip': 'algosec.server.net',
            'username': 'algosec',
            'password': 'admin',
            'ssl_enabled': 'true'
        }
    }


def run_resilient_function(func, *args):
    """
    Run a resilient app function in unit test environment.

    Properly raise exceptions if needed and return the results object.
    :param func:
    :param args:
    :return:
    """
    for message in func(*args):
        if type(message) is FunctionException_:
            raise message
        elif type(message) is FunctionResult:
            return message.value


@pytest.fixture
def options(base_options, additional_options):
    base_options['algosec'].update(additional_options)
    return base_options


@pytest.fixture(scope='session')
def mock_calls_to_resilient_component_class():
    with mock.patch('algosec_resilient.components.algosec_base_component.AlgoSecComponent._call_super'):
        yield


@pytest.fixture
def tested_function(mock_calls_to_resilient_component_class, tested_class, options):
    klass = tested_class(options)
    yield klass._logic



import logging

from algosec.api_clients.business_flow import BusinessFlowAPIClient
from algosec.api_clients.fire_flow import FireFlowAPIClient
from algosec.api_clients.firewall_analyzer import FirewallAnalyzerAPIClient


def str2bool(s):
    return s.lower() in ("yes", "true", "t", "1")


logger = logging.getLogger(__name__)


class AlgosecClient(object):
    ATTRIBUTE_TO_CLIENT_CLASS = {
        '_business_flow': BusinessFlowAPIClient,
        '_fire_flow': FireFlowAPIClient,
        '_firewall_analyzer': FirewallAnalyzerAPIClient,
    }

    def __init__(self, options):
        """
        :param options: The AlgoSec configuration options (should include server, username, password and ssl_enabled
        """
        super(AlgosecClient, self).__init__()
        self._options = options

        # Setting placeholders and parsing the needed options
        self._server_ip = None
        self._username = None
        self._password = None
        self._ssl_enabled = None
        self._parse_connection_details(options)

    def _parse_connection_details(self, options):
        try:
            self._server_ip = options['server_ip']
            self._username = options['username']
            self._password = options['password']
        except KeyError:
            raise Exception("Error while trying to get required configuration settings")

        if 'ssl_enabled' not in options:
            self._ssl_enabled = True
        else:
            self._ssl_enabled = str2bool(options['ssl_enabled'])

    def business_flow(self):
        """
        :rtype: BusinessFlowAPIClient
        """
        return BusinessFlowAPIClient(self._server_ip, self._username, self._password, self._ssl_enabled)

    def fire_flow(self):
        """
        :rtype: FireFlowAPIClient
        """
        return FireFlowAPIClient(self._server_ip, self._username, self._password, self._ssl_enabled)

    def firewall_analyzer(self):
        """
        :rtype: FirewallAnalyzerAPIClient
        """
        return FirewallAnalyzerAPIClient(self._server_ip, self._username, self._password, self._ssl_enabled)



# -*- coding: utf-8 -*-
# pragma pylint: disable=unused-argument, no-self-use
"""Function implementation"""

import logging

from resilient_circuits import function, FunctionResult, StatusMessage

from algosec_resilient.components.algosec_base_component import AlgoSecComponent

logger = logging.getLogger(__name__)


class AlgoSecListAssociatedApplications(AlgoSecComponent):
    """Component that implements Resilient function 'algosec_list_associated_applications"""

    @function("algosec_list_associated_applications")
    def _algosec_list_associated_applications_function(self, event, *args, **kwargs):
        """
        Function: Given an IP/Host list all associated BusinessFlow applications.

        Provides better assessment the risk of the incident. The results contain whether or not it's a critical
        application and a url link to the application on the AlgoSec BusinessFlow dashboard.
        """
        return self.run_login(kwargs)

    def _logic(self, algosec_hostname):
        """The @function decorator offerend by resilient circuits is impossible to unit test..."""
        client = self.algosec.business_flow()
        logger.info("algosec_hostname: %s", algosec_hostname)

        # PUT YOUR FUNCTION IMPLEMENTATION CODE HERE
        yield StatusMessage("starting...")

        associated_applications = [
            {
                'artifact_ip': algosec_hostname,
                'application_name': app_json['name'],
                'is_critical': client.is_application_critical(app_json),
                'businessflow_dashboard': '<a href="{}">{}</a>'.format(
                    client.get_abf_application_dashboard_url(app_json['revisionID']),
                    app_json['name']
                ),
            }
            for app_json in client.get_associated_applications(algosec_hostname)
        ]

        results = {
            'success': True,
            'entries': associated_applications
        }
        yield StatusMessage("done...")

        # Produce a FunctionResult with the results
        yield FunctionResult(results)



import pytest
from mock import patch

from algosec_resilient.components.algosec_list_associated_applications import AlgoSecListAssociatedApplications
from tests.conftest import run_resilient_function


class TestAlgoSecListAssociatedApplications(object):

    @pytest.fixture
    def additional_options(self):
        return {}

    @pytest.fixture
    def tested_class(self):
        return AlgoSecListAssociatedApplications

    @patch(
        'algosec_resilient.components.algosec_client.BusinessFlowAPIClient.get_associated_applications'
    )
    def test_algosec_check_host_internet_connectivity(
            self,
            mock_get_associated_applications,
            tested_function,
    ):
        mock_get_associated_applications.return_value = [
            {'name': 'app1', 'revisionID': 1, 'labels': [{'name': 'Critical'}]},
            {'name': 'app2', 'revisionID': 2},
        ]

        ip = '10.0.0.1'
        result = run_resilient_function(tested_function, ip)

        mock_get_associated_applications.assert_called_once_with(ip)

        app1_dashboard_url = 'https://algosec.server.net/BusinessFlow/#application/1/dashboard'
        app2_dashboard_url = 'https://algosec.server.net/BusinessFlow/#application/2/dashboard'
        assert result == {
            'success': True,
            'entries': [
                {
                    'artifact_ip': ip,
                    'application_name': 'app1',
                    'is_critical': True,
                    'businessflow_dashboard': '<a href="{}">app1</a>'.format(app1_dashboard_url),
                },
                {
                    'artifact_ip': ip,
                    'application_name': 'app2',
                    'is_critical': False,
                    'businessflow_dashboard': '<a href="{}">app2</a>'.format(app2_dashboard_url),
                },
            ]
        }



# -*- coding: utf-8 -*-
"""Abstract base class component for all AlgoSec components"""

from resilient_circuits import ResilientComponent, handler, FunctionError

import algosec_resilient.util.selftest as selftest
from algosec_resilient.components.algosec_client import AlgosecClient


class AlgoSecComponent(ResilientComponent):
    """Abstract base class component for all AlgoSec components"""

    def __init__(self, opts):
        """constructor provides access to the configuration options"""
        self._call_super(opts)
        self.options = opts.get("algosec", {})
        selftest.selftest_function(opts)
        self._setup_algosec_client()

    def _logic(self, algosec_hostname):
        """The @function decorator offerend by resilient circuits is impossible to unit test..."""
        raise NotImplementedError()  # pragma: no cover

    def _call_super(self, opts):
        """
        Allow patching of the super call in unit tests....

        The original ResilientComponent component is super complicated to simply unit test...
        """
        super(AlgoSecComponent, self).__init__(opts)  # pragma: no cover

    def run_login(self, kwargs):
        try:
            for message in self._logic(kwargs['algosec_hostname']):
                yield message
        except Exception:
            yield FunctionError()

    def _setup_algosec_client(self):
        self.algosec = AlgosecClient(self.options)

    @handler("reload")
    def _reload(self, event, opts):
        """Configuration options have changed, save new values"""
        self.options = opts.get("algosec", {})
        self._setup_algosec_client()



# -*- coding: utf-8 -*-
# pragma pylint: disable=unused-argument, no-self-use
"""Function implementation"""

import logging

from algosec.models import ChangeRequestTrafficLine, ChangeRequestAction
from resilient_circuits import function, FunctionResult, StatusMessage

from algosec_resilient.components.algosec_base_component import AlgoSecComponent

logger = logging.getLogger(__name__)


class AlgoSecIsolateHostFromNetwork(AlgoSecComponent):
    """Component that implements Resilient function 'algosec_isolate_host_from_network"""

    @function("algosec_isolate_host_from_network")
    def _algosec_isolate_host_from_network_function(self, event, *args, **kwargs):
        """
        Function: Create a traffic change request with AlgoSec's FireFlow to isolate a host from the network.

        Then AlgoSec's ActiveChange then automatically implements rule changes across all firewalls
        in the network to isolate the host completely.
        """
        return self.run_login(kwargs)

    def _logic(self, algosec_hostname):
        """The @function decorator offerend by resilient circuits is impossible to unit test..."""
        logger.info("algosec_hostname: %s", algosec_hostname)

        # PUT YOUR FUNCTION IMPLEMENTATION CODE HERE
        yield StatusMessage("starting...")
        isolate_traffic_lines = [
            ChangeRequestTrafficLine(
                action=ChangeRequestAction.DROP,
                sources=[algosec_hostname],
                destinations=['*'],
                services=['*'],
            ),
            ChangeRequestTrafficLine(
                action=ChangeRequestAction.DROP,
                sources=['*'],
                destinations=[algosec_hostname],
                services=['*'],
            )
        ]
        try:
            yield StatusMessage("creating isolation change request...")
            change_request_url = self.algosec.fire_flow().create_change_request(
                subject=self.options['isolation_request_subject'].format(algosec_hostname),
                requestor_name=self.options['isolation_request_requestor'],
                email=self.options['isolation_request_requestor_email'],
                traffic_lines=isolate_traffic_lines,
                description=self.options['isolation_request_description'],
                template=self.options.get('isolation_request_template') or None,
            )
        except Exception:
            raise Exception(
                "Error occured while trying to create the isolation change request for {}".format(algosec_hostname)
            )

        yield StatusMessage("done...")

        change_request_id = int(change_request_url.split('=')[1])
        result = {
            'id': change_request_id,
            'hostname': algosec_hostname,
            'url': '<a href="{}">Change Request #{}</a>'.format(change_request_url, change_request_id),
        }

        # Produce a FunctionResult with the result
        yield FunctionResult(result)



# -*- coding: utf-8 -*-
# pragma pylint: disable=unused-argument, no-self-use
"""Function implementation
   test with: resilient-circuits selftest -l algosec
"""

import logging

log = logging.getLogger(__name__)
log.setLevel(logging.INFO)
log.addHandler(logging.StreamHandler())


def selftest_function(opts):
    """
    Placeholder for selftest function. An example use would be to test package api connectivity.
    Suggested return values are be unimplemented, success, or failure.
    """
    options = opts.get("algosec", {})  # NOQA
    return {"state": "unimplemented"}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

# Declare the Resilient Function components. Dict from Class name to module name in the components directory.
FUNCTION_COMPONENTS = {
    'AlgoSecIsolateHostFromNetwork': 'algosec_isolate_host_from_network',
    'AlgoSecCheckHostInternetConnectivity': 'algosec_check_host_internet_connectivity',
    'AlgoSecListAssociatedApplications': 'algosec_list_associated_applications',

}

setup(
    name='algosec_resilient',
    version='1.3.0',
    license='MIT',
    author='AlgoSec',
    author_email='support@algosec.com',
    url='https://www.algosec.com/',
    description="Resilient Circuits Components For The AlgoSec Integration",
    long_description="Resilient Circuits Components For The AlgoSec Integration",
    install_requires=[
        'resilient>=31.0.0',
        'resilient_circuits>=31.0.0',
        'algosec>=1.3.0',
    ],
    packages=find_packages(),
    include_package_data=True,
    platforms='any',
    classifiers=[
        'Programming Language :: Python',
    ],
    entry_points={
        "resilient.circuits.components": [
            '{}FunctionComponent = algosec_resilient.components.{}:{}'.format(klass, module, klass)
            for klass, module in FUNCTION_COMPONENTS.items()
        ],
        "resilient.circuits.configsection": ["gen_config = algosec_resilient.util.config:config_section_data"],
        "resilient.circuits.customize": ["customize = algosec_resilient.util.customize:customization_data"],
        "resilient.circuits.selftest": ["selftest = algosec_resilient.util.selftest:selftest_function"]
    }
)



import mock
import pytest
from algosec.models import ChangeRequestAction
from mock import ANY

from algosec_resilient.components.algosec_isolate_host_from_network import AlgoSecIsolateHostFromNetwork
from tests.conftest import run_resilient_function


class TestAlgoSecIsolateHostFromNetwork(object):

    @pytest.fixture
    def additional_options(self):
        return {
            'isolation_request_subject': 'Example Request Subject For {}',
            'isolation_request_requestor': 'Example Requestor',
            'isolation_request_requestor_email': 'resilient@integration.dummy',
            'isolation_request_description': 'Example Change Request Description',
            'isolation_request_template': 'Example Template',
        }

    @pytest.fixture
    def tested_class(self):
        return AlgoSecIsolateHostFromNetwork

    @mock.patch('algosec_resilient.components.algosec_client.FireFlowAPIClient.create_change_request')
    def test_algosec_check_host_internet_connectivity(
            self,
            mock_create_change_request,
            additional_options,
            tested_function,
    ):
        change_request_id = 1234567890
        change_request_url = 'some-change-request-url=1234567890'
        mock_create_change_request.return_value = change_request_url

        ip = '10.0.0.1'

        result = run_resilient_function(tested_function, ip)

        mock_create_change_request.assert_called_once_with(
            subject=additional_options['isolation_request_subject'].format(ip),
            requestor_name=additional_options['isolation_request_requestor'],
            email=additional_options['isolation_request_requestor_email'],
            traffic_lines=ANY,
            description=additional_options['isolation_request_description'],
            template=additional_options.get('isolation_request_template') or None,
        )

        assert result == {
            'id': change_request_id,
            'hostname': ip,
            'url': '<a href="{}">Change Request #{}</a>'.format(change_request_url, change_request_id),
        }

    @mock.patch('algosec_resilient.components.algosec_client.FireFlowAPIClient.create_change_request')
    def test_algosec_check_host_internet_connectivity__check_traffic_lines(
            self,
            mock_create_change_request,
            additional_options,
            tested_function,
    ):
        ip = '10.0.0.1'

        run_resilient_function(tested_function, ip)

        for traffic_line in mock_create_change_request.call_args[1]['traffic_lines']:
            assert traffic_line.action == ChangeRequestAction.DROP
            assert traffic_line.services == ['*']
            assert (
                    (traffic_line.sources == [ip] and traffic_line.destinations == ['*'])
                    or
                    (traffic_line.sources == ['*'] and traffic_line.destinations == [ip])
            )



# -*- coding: utf-8 -*-
# pragma pylint: disable=unused-argument, no-self-use
"""Function implementation"""

import logging

from algosec.models import DeviceAllowanceState
from resilient_circuits import function, StatusMessage, FunctionResult

from algosec_resilient.components.algosec_base_component import AlgoSecComponent

logger = logging.getLogger(__name__)


class AlgoSecCheckHostInternetConnectivity(AlgoSecComponent):
    """Component that implements Resilient function 'algosec_check_host_internet_connectivity"""

    @function("algosec_check_host_internet_connectivity")
    def _algosec_check_host_internet_connectivity_function(self, event, *args, **kwargs):
        """
        Function: Given a hostname, return whether or not it has internet access.

        The AlgoSec integration will check if a given host/IP is has access to a public
        known internet node such as 8.8.8.8.
        """
        return self.run_login(kwargs)

    def _logic(self, algosec_hostname):
        """The @function decorator offerend by resilient circuits is impossible to unit test..."""
        logger.info("algosec_hostname: %s", algosec_hostname)

        # PUT YOUR FUNCTION IMPLEMENTATION CODE HERE
        yield StatusMessage("starting...")

        query_dict = self.algosec.firewall_analyzer().execute_traffic_simulation_query(
            algosec_hostname,
            self.options['internet_connectivity_check_external_ip'],
            self.options['internet_connectivity_check_service'],
        )

        query_result, query_url = query_dict['result'], query_dict['query_url']

        if query_result in (DeviceAllowanceState.ALLOWED, DeviceAllowanceState.PARTIALLY_BLOCKED):
            is_it_connected_to_the_internet = 'Yes'
        elif query_result == DeviceAllowanceState.NOT_ROUTED:
            is_it_connected_to_the_internet = 'No'
        else:
            is_it_connected_to_the_internet = 'No'

        connectivity_result = {
            'success': True,
            'artifact_ip': algosec_hostname,
            'is_it_connected_to_the_internet': is_it_connected_to_the_internet,
            'query_url': '<a href="{}">Query Results</a>'.format(query_url),
        }

        yield StatusMessage("done...")

        # Produce a FunctionResult with the results
        yield FunctionResult(connectivity_result)
